﻿using System;
using System.Collections;
using System.Diagnostics;
using System.Linq;
using System.Windows;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.DragDrop;
using Telerik.Windows.Controls.TreeView;
using Telerik.Windows.Documents;
using Telerik.Windows.Documents.Model;

namespace RadRichTextDragDrop
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.richTextBox.AllowDrop = true;

            RadDragAndDropManager.AddDropQueryHandler(this.richTextBox, new EventHandler<DragDropQueryEventArgs>(RichTextBoxOnDropQuery));
            RadDragAndDropManager.AddDropInfoHandler(this.richTextBox, new EventHandler<DragDropEventArgs>(RichTextBoxOnDropInfo));
            RadDragAndDropManager.AddDropQueryHandler(treeView, new EventHandler<DragDropQueryEventArgs>(TreeOnDropQuery));
            RadDragAndDropManager.AddDragInfoHandler(treeView, TreeOnDragInfo);
        }

        private void TreeOnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            e.QueryResult = true;
            e.Handled = true;
        }

        private void RichTextBoxOnDropQuery(object sender, DragDropQueryEventArgs e)
        {
            if (e.Options.Status == DragStatus.DropDestinationQuery)
            {
                e.Options.ArrowCue = RadDragAndDropManager.GenerateArrowCue();
            }

            e.QueryResult = true;
            e.Handled = true;
        }

        private void TreeOnDragInfo(object sender, DragDropEventArgs e)
        {
            TreeViewDragCue cue = e.Options.DragCue as TreeViewDragCue;
            cue.IsDropPossible = e.Options.Destination is RadRichTextBox;
        }

        private void RichTextBoxOnDropInfo(object sender, DragDropEventArgs e)
        {
            marker.Visibility = Visibility.Collapsed;
            if (e.Options.Status == DragStatus.DropPossible)
            {
                RadRichTextBox rrtb = e.Options.Destination as RadRichTextBox;
                Point viewPoint = e.Options.RelativeDragPoint;
                Debug.WriteLine("viewpoint: " + viewPoint.X + " " + viewPoint.Y);
                Point adjustedPoint = new Point(viewPoint.X - 20, viewPoint.Y - 20); //adjust the point a bit, so that the cue does not hide the drop position marker
                DocumentPosition position = rrtb.ActiveEditorPresenter.GetDocumentPositionFromViewPoint(adjustedPoint);
                Point documentPoint = rrtb.ActiveEditorPresenter.GetViewPointFromDocumentPosition(position);
                Debug.WriteLine("documentPoint: " + documentPoint.X + " " + documentPoint.Y);
                rrtb.Document.CaretPosition.MoveToPosition(position);
                double? fontSize = (rrtb.CurrentEditingStyle.GetPropertyValue(Span.FontSizeProperty) as double?);
                if (fontSize == null)
                {
                    fontSize = rrtb.Document.StyleRepository["Normal"].SpanProperties.FontSize;
                }
                this.marker.Height = 1.5 * fontSize.Value * rrtb.ScaleFactor.Height;
                marker.Visibility = Visibility.Visible;
                marker.Margin = new Thickness(documentPoint.X + rrtb.Margin.Left, documentPoint.Y + rrtb.Margin.Top, 0, 0);
            }

            if (e.Options.Status == DragStatus.DropComplete)
            {
                RadRichTextBox rrtb = e.Options.Destination as RadRichTextBox;

                ICollection items = e.Options.Payload as ICollection;
                rrtb.Insert(items.OfType<object>().First().ToString());
            }
        }
    }
}
